unit clsComObjectWithEvents;

interface

{$INCLUDE SvCom.inc}

uses
    Windows,
    Classes,
    ComObj,
    SvCom_ComObj,
    SvCom_NTService,
    SvCom_ComService,
    AxCtrls,
    ActiveX,
    ComServiceWithEvents_TLB, StdVcl, SvCom_WinSvc;

type
  TComObjectWithEvents = class(TAutoObject, IConnectionPointContainer, IComObjectWithEvents)
  private
    { Private declarations }
    FConnectionPoints: TConnectionPoints;
    FConnectionPoint: TConnectionPoint;
    FEvents: IComObjectWithEventsEvents;
    { note: FEvents maintains a *single* event sink. For access to more
      than one event sink, use FConnectionPoint.SinkList, and iterate
      through the list of sinks. }
  protected
    procedure RaiseEvent(const Text: WideString); safecall;
    { Protected declarations }
  public
    { Public declarations }
    procedure Initialize; override;
    destructor Destroy; override;
    property ConnectionPoints: TConnectionPoints read FConnectionPoints
      implements IConnectionPointContainer;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
  end;

type
    TComObjectWithEventsSingletoneFactory = class(TSvcAutoObjectFactory, IClassFactory)
        function CreateInstance(const UnkOuter: IUnknown; const IID: TGUID; out Obj): HResult; stdcall;
    end;

var ComObjectWithEvents: TComObjectWithEvents;

implementation

uses SvCom_ComServ, Math;

procedure TComObjectWithEvents.EventSinkChanged(const EventSink: IUnknown);
begin
    SvSendMethodEnter('EventSinkChanged');
    FEvents := EventSink as IComObjectWithEventsEvents;
    SvSendMethodLeave('EventSinkChanged');
end;

procedure TComObjectWithEvents.Initialize;
begin
    inherited;
    if ComObjectWithEvents = nil then ComObjectWithEvents := Self;
    FConnectionPoints := TConnectionPoints.Create(Self);
    if AutoFactory.EventTypeInfo <> nil then
        FConnectionPoint := FConnectionPoints.CreateConnectionPoint(
          AutoFactory.EventIID, ckMulti, EventConnect)
    else
        FConnectionPoint := nil;
end;

destructor TComObjectWithEvents.Destroy;
begin
    if ComObjectWithEvents = Self then ComObjectWithEvents := nil;
    inherited;
end;

// Implementation of TComObjectWithEvents singletone factory
function TComObjectWithEventsSingletoneFactory.CreateInstance(const UnkOuter: IUnknown; const IID: TGUID; out Obj): HResult;
var ComObjectWithEventsExists: Boolean;
begin
    if (ComObjectWithEvents = nil) or (UnkOuter <> nil) then
    begin
        ComObjectWithEventsExists := ComObjectWithEvents <> nil;
        result:=inherited CreateInstance(UnkOuter,IID,Obj);
        if  (UnkOuter <> nil) and not ComObjectWithEventsExists then ComObjectWithEvents := nil;
    end else
        result:=ComObjectWithEvents.QueryInterface(IID,Obj);
end;

procedure TComObjectWithEvents.RaiseEvent(const Text: WideString);
var i: Integer;
    U: IUnknown;
    E: IComObjectWithEventsEvents;
begin
    with FConnectionPoint.SinkList do
    begin
        for i:=0 to Count-1 do
        begin
            U:=IUnknown(Items[i]);
            if not Assigned(U) then continue;
            E:=U as IComObjectWithEventsEvents;
            E.HandleEventNotification('Event is raised with message: '+Text);
        end;
    end;
end;

initialization
    TComObjectWithEventsSingletoneFactory.Create(SvcComServer, TComObjectWithEvents, Class_ComObjectWithEvents,
        ciMultiInstance, 'ComServiceWithEvents');
end.

